@@ -1,7 +1,7 @@
 import os.path
 import sys
 from pathlib import Path
-from typing import List, Dict, Tuple
+from typing import List, Dict, Tuple, Union
 
 from bigeye_sdk.bigconfig_validation.big_config_reports import raise_files_contain_error_exception, MetricSuiteReport, \
     process_reports, ProcessStage
@@ -17,7 +17,7 @@ from bigeye_sdk.generated.com.bigeye.models.generated import Source, CohortDefin
     MetricSuite, CohortAndMetricDefinition, CatalogEntityType, MetricDefinition, FieldType
 from bigeye_sdk.model.big_config import BigConfig, RowCreationTimes, TagDeployment, TableDeployment
 from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName
-from bigeye_sdk.model.protobuf_message_facade import SimpleSLA, SimpleMetricDefinition
+from bigeye_sdk.model.protobuf_message_facade import SimpleCollection, SimpleMetricDefinition, SimpleSLA
 from bigeye_sdk.serializable import File, BIGCONFIG_FILE
 
 
@@ -93,7 +93,7 @@ class MetricSuiteController:
     def _upsert_slas(self,
                      bigconfig: BigConfig,
                      overwrite: bool,
-                     apply=False) -> List[SimpleSLA]:
+                     apply=False) -> List[Union[SimpleSLA, SimpleCollection]]:
         """
         Currently operates as an overwrite of existing metrics.  This prevents altering in the front end.  Every time the
         config is run it will overwrite what is already there.  Does not add metrics.
@@ -106,9 +106,9 @@ class MetricSuiteController:
         existing_slas = {sla.name: sla for sla in self.client.get_collections().collections}
         deployment_slas = []
 
-        for sla in bigconfig.get_slas():
+        for sla in bigconfig.get_collections():
             if sla.name in existing_slas.keys():
-                existing = SimpleSLA.from_datawatch_object(existing_slas[sla.name])
+                existing = SimpleCollection.from_datawatch_object(existing_slas[sla.name])
                 merged = sla.merge_for_upsert(existing=existing, overwrite=overwrite)
                 deployment_slas.append(merged)
                 if apply:
@@ -122,7 +122,7 @@ class MetricSuiteController:
                     notification_channels=c.notification_channels,
                     muted_until_timestamp=c.muted_until_timestamp
                 ).collection
-                deployment_slas.append(SimpleSLA.from_datawatch_object(c))
+                deployment_slas.append(SimpleCollection.from_datawatch_object(c))
 
         return deployment_slas
 
@@ -193,12 +193,12 @@ class MetricSuiteController:
 
     def tag_deployment_to_cohort_and_metric_def(self,
                                                 tag_deployment: TagDeployment,
-                                                deployment_sla: SimpleSLA = None
+                                                deployment_sla: Union[SimpleSLA, SimpleCollection] = None
                                                 ) -> Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TagDeployment object.   For table level metrics
         Args:
-            tag_deployment: tag deployment to convert to cohort and metric definitoin.
+            tag_deployment: tag deployment to convert to cohort and metric definition.
             deployment_sla: The sla associated with the deployment.
 
         Returns: Dict[source_id: int, List[CohortAndMetricDefinition]] matched based on source_id to consolidate to
@@ -211,8 +211,8 @@ class MetricSuiteController:
 
         for m in tag_deployment.metrics:
             """segregate table level metrics from column level and add slas"""
-            if deployment_sla and deployment_sla.id not in m.sla_ids:
-                m.sla_ids.append(deployment_sla.id)
+            if deployment_sla and deployment_sla.id not in m.collection_ids:
+                m.collection_ids.append(deployment_sla.id)
             mdwo = m.to_datawatch_object()
             if _is_table_level_metric(metric_type=m.metric_type, table_level_metrics=self.table_level_metrics):
                 mdwo.is_table_metric = True
@@ -281,7 +281,7 @@ class MetricSuiteController:
 
     def table_deployment_to_cohort_and_metric_def(self,
                                                   table_deployment: TableDeployment,
-                                                  deployment_sla: SimpleSLA = None) -> Dict[
+                                                  deployment_sla: Union[SimpleSLA, SimpleCollection] = None) -> Dict[
         int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TableDeployment object
@@ -360,8 +360,8 @@ class MetricSuiteController:
                         error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg)
                     )
                 else:
-                    if deployment_sla and deployment_sla.id not in m.sla_ids:
-                        m.sla_ids.append(deployment_sla.id)
+                    if deployment_sla and deployment_sla.id not in m.collection_ids:
+                        m.collection_ids.append(deployment_sla.id)
                     mdwo = m.to_datawatch_object()
                     col_metrics.append(mdwo)
 
@@ -376,7 +376,7 @@ class MetricSuiteController:
 
     def bigconfig_to_metric_suites(self,
                                    bigconfig: BigConfig,
-                                   deployment_slas: Dict[str, SimpleSLA]) -> List[MetricSuite]:
+                                   deployment_slas: Dict[str, Union[SimpleSLA, SimpleCollection]]) -> List[MetricSuite]:
         """
         Creates a MetricSuite for each source identified in a Bigconfig Table or Tag Deployment.
         Args:
@@ -436,7 +436,7 @@ class MetricSuiteController:
                         rct_cohorts[sid] = [cohorts]
 
         deployment_metric_suites: Dict[int, MetricSuite] = {
-            source_id: MetricSuite(source_id=source_id, definitions=definitions)
+            source_id: MetricSuite(source_id=source_id, definitions=definitions, auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
             for source_id, definitions in cmds.items()
         }
 
@@ -449,8 +449,9 @@ class MetricSuiteController:
             metric_suites.append(metric_suite)
 
         for source_id, rcts in rct_cohorts.items():
+            #TODO: Will need to refactor when supporting multiple files
             metric_suites.append(
-                MetricSuite(source_id=source_id, row_creation_cohorts=rcts)
+                MetricSuite(source_id=source_id, row_creation_cohorts=rcts, auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
             )
 
         return metric_suites
