@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from typing import List, Optional, Dict, Tuple, Any, Union
 
+
 import yaml
 from pydantic import Field, PrivateAttr, validator, root_validator
 
@@ -11,10 +12,10 @@ from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import DUPLI
     DUPLICATE_TAG_EXISTS_ERRMSG, TAG_ID_NOT_EXISTS_IN_TAG_DEFINITION_ERRMSG, \
     SAVED_METRIC_ID_NOT_EXISTS_IN_SAVED_METRICS_DEFINITION_ERRMSG, FQ_COL_NOT_RESOLVES_TO_COLUMN_ERRMSG, \
     WILD_CARDS_NOT_SUPPORT_IN_FQ_TABLE_NAMES_ERRMSG, FQ_TABLE_NAME_MUST_RESOLVE_TO_TABLE_ERRMSG
-from bigeye_sdk.functions.bigconfig_functions import explode_fq_name, explode_fq_table_name
+from bigeye_sdk.functions.bigconfig_functions import explode_fq_name, explode_fq_table_name, explode_fq_column_selectors
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.protobuf_enum_facade import SimpleFieldType
-from bigeye_sdk.model.protobuf_message_facade import SimpleMetricDefinition, SimpleSLA
+from bigeye_sdk.model.protobuf_message_facade import SimpleMetricDefinition, SimpleCollection, SimpleSLA
 from bigeye_sdk.serializable import BigConfigFile
 
 log = get_logger(__file__)
@@ -135,9 +136,23 @@ class TagDeployment(YamlModelWithValidatorContext):
         return values
 
 
+    def explode_fq_column_selectors(self) -> List[List[str]]:
+        """
+        Explodes a fully qualified column selectors into a list of names.  Supports wild cards as *. Supports single and
+        double-quoted  names containing periods.  Supports fully qualified names with either source.database.schema or
+        source.schema convention.
+
+            Example: wh."my.*".some_*_table.* resolves to ['wh', 'my.*', 'some_*_table', '*']
+
+        Returns: list of names from each fully qualified column selector in the Tag Deployment
+        """
+        return [explode_fq_column_selectors(i.name) for i in self.column_selectors]
+
+
 class TagDeploymentSuite(YamlModelWithValidatorContext):
     deployments: List[TagDeployment]
     sla: Optional[SimpleSLA] = None
+    collection: Optional[SimpleCollection] = None
 
     @root_validator(pre=True)
     def must_be_list(cls, values):
@@ -152,6 +167,14 @@ class TagDeploymentSuite(YamlModelWithValidatorContext):
 
         return v
 
+    @root_validator(pre=True)
+    def warn_and_use_collection(cls, values):
+        if values.get('sla'):
+            log.warning('sla class variable is deprecated and will be removed in future versions. Use collection.')
+            values['collection'] = values['sla']
+
+        return values
+
 
 class ColumnMetricDeployment(YamlModelWithValidatorContext):
     column_name: str
@@ -208,8 +231,8 @@ class TableDeployment(YamlModelWithValidatorContext):
 
     def explode_fq_table_name(self):
         """
-        Explodes a fully qualifeid table name into a list of names.  Supports single and double quoted  names
-        containing periods.  Supports fully qulified names with either source.database.schema or source.schema
+        Explodes a fully qualified table name into a list of names.  Supports single and double-quoted  names
+        containing periods.  Supports fully qualified names with either source.database.schema or source.schema
         conventions.  DOES NOT support wild cards.
 
             Example: wh."my.schema".some_table resolves to ['wh', 'my.schema', 'some_table']
@@ -223,6 +246,7 @@ class TableDeployment(YamlModelWithValidatorContext):
 class TableDeploymentSuite(YamlModelWithValidatorContext):
     deployments: List[TableDeployment]
     sla: Optional[SimpleSLA] = None
+    collection: Optional[SimpleCollection] = None
 
     @root_validator(pre=True)
     def must_be_list(cls, values):
@@ -236,6 +260,14 @@ class TableDeploymentSuite(YamlModelWithValidatorContext):
             m.deployment_validations(config_error_lines=v.get_error_lines())
         return v
 
+    @root_validator(pre=True)
+    def warn_and_use_collection(cls, values):
+        if values.get('sla'):
+            log.warning('sla class variable is deprecated and will be removed in future versions. Use collection.')
+            values['collection'] = values['sla']
+
+        return values
+
 
 class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
     """
@@ -243,6 +275,7 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
     into a single object that can be used to generate a metric suite.  Tag Definitions and Saved Metric Definitions
     are applied -- and validated -- during the __post_init__ phase of instantiating a Bigconfig.
     """
+    auto_apply_on_indexing: bool = False
     tag_definitions: Optional[List[TagDefinition]] = Field(
         default_factory=lambda: [])  # only one because we must consolidate if creating Bigconfig from multiple files.
     row_creation_times: Optional[RowCreationTimes] = RowCreationTimes()
@@ -371,7 +404,7 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
                     tag_deps: List[TagDeploymentSuite],
                     row_creation_times: RowCreationTimes) -> Tuple[List[TagDeploymentSuite], RowCreationTimes]:
         """
-        Applies tags by tag id in all tag deployments and row creation times definitinos.  Validates that all tags
+        Applies tags by tag id in all tag deployments and row creation times definitions.  Validates that all tags
         called in deployments exist in the tags definitions.  Validates that column selectors exist after application.
         Args:
             tag_ix: index of column selectors keyed by tag_id
@@ -407,7 +440,7 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
 
             for attr in override_smd.__dict__.keys():
                 override_attr_value = getattr(override_smd, attr)
-                if attr not in ['saved_metric_id', 'metric_type', 'sla_ids'] \
+                if attr not in ['saved_metric_id', 'metric_type', 'sla_ids', 'collection_ids'] \
                         and not SimpleMetricDefinition.is_default_value(attr, override_attr_value):
                     setattr(r, attr, override_attr_value)
 
@@ -454,15 +487,15 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
 
         return tag_deps, table_deps
 
-    def get_slas(self) -> List[SimpleSLA]:
-        slas: List[SimpleSLA] = []
+    def get_collections(self) -> List[SimpleCollection]:
+        collections: List[SimpleCollection] = []
 
         for d in self.tag_deployments:
-            if d.sla:
-                slas.append(d.sla)
+            if d.collection:
+                collections.append(d.collection)
 
         for d in self.table_deployments:
-            if d.sla:
-                slas.append(d.sla)
+            if d.collection:
+                collections.append(d.collection)
 
-        return slas
+        return collections
