@@ -6,6 +6,7 @@ from typing import List, Optional, TypeVar, Union
 
 import betterproto
 import yaml
+from deprecated import deprecated
 from pydantic import validator, Field, root_validator
 from pydantic_yaml import YamlStrEnum
 
@@ -44,7 +45,8 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
     def get_freshness_metric_types(cls) -> List[SimplePredefinedMetric]:
         freshness_metric_names = [SimplePredefinedMetricName.HOURS_SINCE_LAST_LOAD,
                                   SimplePredefinedMetricName.HOURS_SINCE_MAX_DATE,
-                                  SimplePredefinedMetricName.HOURS_SINCE_MAX_TIMESTAMP]
+                                  SimplePredefinedMetricName.HOURS_SINCE_MAX_TIMESTAMP,
+                                  SimplePredefinedMetricName.FRESHNESS]
         return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in freshness_metric_names]
 
     @classmethod
@@ -113,7 +115,7 @@ class SimpleTemplateMetric(SimpleMetricType, type='TEMPLATE'):
 
     def to_datawatch_object(self, **kwargs) -> MetricType:
         return MetricType(template_metric=TemplateMetric(template_id=self.template_id,
-                                                         aggregation_type=self.aggregation_type.to_protpbuf(),
+                                                         aggregation_type=self.aggregation_type.to_datawatch_object(),
                                                          template_name=self.template_name))
 
 
@@ -206,8 +208,8 @@ class SimpleThreshold(PydanticSubtypeSerializable, DatawatchFacade, ABC):
             )
             return sat
         elif t == 'constant_threshold':
-            lower_bound = 0.0
-            upper_bound = 0.0
+            lower_bound = None
+            upper_bound = None
             for i in obj:
                 if i.constant_threshold.bound.bound_type == SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE:
                     lower_bound = i.constant_threshold.bound.value
@@ -291,8 +293,8 @@ class SimpleAutoThreshold(SimpleThreshold, type='AUTO'):
 
 
 class SimpleConstantThreshold(SimpleThreshold, type='CONSTANT'):
-    upper_bound: Optional[float] = 0
-    lower_bound: Optional[float] = 0
+    upper_bound: Optional[float] = None
+    lower_bound: Optional[float] = None
 
     def to_datawatch_object(self) -> List[Threshold]:
         """
@@ -300,12 +302,14 @@ class SimpleConstantThreshold(SimpleThreshold, type='CONSTANT'):
         :return: a List of Thresholds
         """
         thresholds: List[Threshold] = []
-        lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
-        lbt = Threshold(constant_threshold=ConstantThreshold(bound=lb))
-        thresholds.append(lbt)
-        ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
-        ubt = Threshold(constant_threshold=ConstantThreshold(bound=ub))
-        thresholds.append(ubt)
+        if self.lower_bound is not None:
+            lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
+            lbt = Threshold(constant_threshold=ConstantThreshold(bound=lb))
+            thresholds.append(lbt)
+        if self.upper_bound is not None:
+            ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
+            ubt = Threshold(constant_threshold=ConstantThreshold(bound=ub))
+            thresholds.append(ubt)
 
         return thresholds
 
@@ -329,38 +333,42 @@ class SimpleFreshnessThreshold(SimpleThreshold, type='FRESHNESS'):
 
 class SimpleRelativeThreshold(SimpleThreshold, type='RELATIVE'):
     lookback: SimpleTimeInterval
-    upper_bound: Optional[float] = 0
-    lower_bound: float = 0
+    upper_bound: Optional[float] = None
+    lower_bound: Optional[float] = None
 
     def to_datawatch_object(self, **kwargs) -> List[Threshold]:
         lkbk = self.lookback.to_datawatch_object()
 
         thresholds: List[Threshold] = []
-        lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
-        lbt = Threshold(relative_threshold=RelativeThreshold(lookback=lkbk, bound=lb))
-        thresholds.append(lbt)
-        ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
-        ubt = Threshold(relative_threshold=RelativeThreshold(lookback=lkbk, bound=ub))
-        thresholds.append(ubt)
+        if self.lower_bound is not None:
+            lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
+            lbt = Threshold(relative_threshold=RelativeThreshold(lookback=lkbk, bound=lb))
+            thresholds.append(lbt)
+        if self.upper_bound is not None:
+            ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
+            ubt = Threshold(relative_threshold=RelativeThreshold(lookback=lkbk, bound=ub))
+            thresholds.append(ubt)
 
         return thresholds
 
 
 class SimpleStdDevThreshold(SimpleThreshold, type='STDDEV'):
     lookback: SimpleTimeInterval
-    upper_bound: Optional[float] = 0
-    lower_bound: float = 0
+    upper_bound: Optional[float] = None
+    lower_bound: Optional[float] = None
 
     def to_datawatch_object(self, **kwargs) -> List[Threshold]:
         lkbk = self.lookback.to_datawatch_object()
 
         thresholds: List[Threshold] = []
-        lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
-        lbt = Threshold(standard_deviation_threshold=StandardDeviationThreshold(lookback=lkbk, bound=lb))
-        thresholds.append(lbt)
-        ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
-        ubt = Threshold(standard_deviation_threshold=StandardDeviationThreshold(lookback=lkbk, bound=ub))
-        thresholds.append(ubt)
+        if self.lower_bound is not None:
+            lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=self.lower_bound)
+            lbt = Threshold(standard_deviation_threshold=StandardDeviationThreshold(lookback=lkbk, bound=lb))
+            thresholds.append(lbt)
+        if self.upper_bound is not None:
+            ub = SimpleBound(bound_type=SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE, value=self.upper_bound)
+            ubt = Threshold(standard_deviation_threshold=StandardDeviationThreshold(lookback=lkbk, bound=ub))
+            thresholds.append(ubt)
 
         return thresholds
 
@@ -447,7 +455,7 @@ class SimpleMetricParameter(YamlModelWithValidatorContext, DatawatchFacade):
                                column_name=self.column_name,
                                number_value=self.number_value)
 
-
+@deprecated('SimpleSLA is deprecated and will be removed in future versions. Use SimpleCollection instead.')
 class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
     name: str
     id: int = None
@@ -465,7 +473,7 @@ class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
 
     def merge_for_upsert(self, existing: SimpleSLA, overwrite: bool = True) -> SimpleSLA:
         """
-        Merges this SLA and existing SLA.  Attributes that will be merged include: notificaiton channels and metric ids.
+        Merges this SLA and existing SLA.  Attributes that will be merged include: notification channels and metric ids.
 
         If not overwriting: missing names, descriptions, and muted_until_timestamp will be populated with
         existing values.
@@ -485,7 +493,7 @@ class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
         new_sla.id = existing.id
 
         if not overwrite:
-            """If we are not overwriting then we want to merge notifivation channels and metric ids."""
+            """If we are not overwriting then we want to merge notification channels and metric ids."""
             if not new_sla.name:
                 new_sla.name = existing.name
 
@@ -534,7 +542,96 @@ class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
         )
 
         return c
+    
+    def to_simple_collection(self) -> SimpleCollection:
+        return SimpleCollection.parse_obj(self)
 
+class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
+    name: str
+    id: int = None
+    description: str = None
+    metric_ids: List[int] = None
+    notification_channels: List[SimpleNotificationChannel] = Field(default_factory=lambda: [])
+    muted_until_timestamp: int = 0
+
+    @root_validator(pre=True)
+    def must_be_list(cls, values):
+        must_be_list_validator(clazz=SimpleCollection, attribute_name='metric_ids', values=values)
+        must_be_list_validator(clazz=SimpleCollection, attribute_name='notification_channels', values=values)
+
+        return values
+
+    def merge_for_upsert(self, existing: SimpleCollection, overwrite: bool = True) -> SimpleCollection:
+        """
+        Merges this collection and existing collection.  Attributes that will be merged include: notification channels and metric ids.
+
+        If not overwriting: missing names, descriptions, and muted_until_timestamp will be populated with
+        existing values.
+
+        If overwriting: (current default strategy) new attributes will replace the old -- even if empty.
+
+        Args:
+            existing: the existing collection to merge this new collection with.
+            overwrite: true/false
+
+        Returns: The merged collection
+
+        """
+        new_collection = self.copy(deep=True)
+
+        # set existing SLA ID.
+        new_collection.id = existing.id
+
+        if not overwrite:
+            """If we are not overwriting then we want to merge notification channels and metric ids."""
+            if not new_collection.name:
+                new_collection.name = existing.name
+
+            if not new_collection.description:
+                new_collection.description = existing.description
+
+            if not new_collection.muted_until_timestamp:
+                new_collection.muted_until_timestamp = existing.muted_until_timestamp
+
+            if new_collection.notification_channels:
+                new_collection.notification_channels.extend(existing.notification_channels)
+            else:
+                new_collection.notification_channels = existing.notification_channels
+
+            if new_collection.metric_ids:
+                new_collection.metric_ids.extend(existing.metric_ids)
+            else:
+                new_collection.metric_ids = existing.metric_ids
+
+        return new_collection
+
+    @classmethod
+    def from_datawatch_object(cls, obj: Collection) -> SimpleCollection:
+        collection = SimpleCollection(
+            id=obj.id,
+            name=obj.name,
+            description=obj.description,
+            metric_ids=obj.metric_ids,
+            muted_until_timestamp=obj.muted_until_timestamp
+        )
+
+        if obj.notification_channels:
+            collection.notification_channels = [SimpleNotificationChannel.from_datawatch_object(nc)
+                                         for nc in obj.notification_channels]
+
+        return collection
+
+    def to_datawatch_object(self) -> Collection:
+        c = Collection(
+            id=self.id,
+            name=self.name,
+            description=self.description if self.description else "SDK Generated.",
+            metric_ids=self.metric_ids,
+            notification_channels=[nc.to_datawatch_object() for nc in self.notification_channels],
+            muted_until_timestamp=self.muted_until_timestamp
+        )
+
+        return c
 
 class BucketSize(YamlStrEnum):
     DAY = "DAY"  # 86400
@@ -579,6 +676,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     lookback: Optional[SimpleLookback] = SimpleLookback()
     muted_until_epoch_seconds: Optional[int] = 0
     sla_ids: Optional[List[int]] = Field(default_factory=lambda: [])
+    collection_ids: Optional[List[int]] = Field(default_factory=lambda: [])
 
     @validator('threshold', always=True, pre=False)
     def set_default_threshold(cls, threshold, values):
@@ -659,6 +757,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
     @root_validator(pre=True)
     def must_be_list(cls, values):
+        must_be_list_validator(clazz=SimpleMetricDefinition, attribute_name='collection_ids', values=values)
         must_be_list_validator(clazz=SimpleMetricDefinition, attribute_name='sla_ids', values=values)
         must_be_list_validator(clazz=SimpleMetricDefinition, attribute_name='conditions', values=values)
         must_be_list_validator(clazz=SimpleMetricDefinition, attribute_name='group_by', values=values)
@@ -666,11 +765,19 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         return values
 
+    @root_validator(pre=True)
+    def warn_and_use_collection(cls, values):
+        if values.get('sla_ids'):
+            log.warning('sla_id class variable is deprecated and will be removed in future versions. Use collection_ids.')
+            values['collection_ids'] = values['sla_ids']
+
+        return values
+
     def deployment_validations(self, config_error_lines: List[str]):
         """
         Runs all deployment validations for SimpleMetricDefinitions
         Args:
-            config_error_lines: The congiruation serialized to yaml and split to lines.
+            config_error_lines: The configuration serialized to yaml and split to lines.
 
         Returns: None
         """
@@ -682,7 +789,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         Metric definitions must have a metric type if not referencing a saved metric by id
         (deployment in-line metrics)
         Args:
-            config_error_lines: The congiruation serialized to yaml and split to lines.
+            config_error_lines: The configuration serialized to yaml and split to lines.
 
         Returns: None
         """
@@ -696,7 +803,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         """Metric definitions cannot have a metric_ type if referencing a saved_metric_id but no metric_type
             (deployment reference of saved metric)
         Args:
-            config_error_lines: The congiruation serialized to yaml and split to lines.
+            config_error_lines: The configuration serialized to yaml and split to lines.
 
         Returns: None
         """
@@ -745,7 +852,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         builder.muted_until_epoch_seconds = obj.muted_until_epoch_seconds
 
-        builder.sla_ids = obj.collection_ids
+        builder.collection_ids = obj.collection_ids
 
         return builder
 
@@ -788,7 +895,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         builder.muted_until_epoch_seconds = self.muted_until_epoch_seconds
 
-        if self.sla_ids:
-            builder.collection_ids = self.sla_ids
+        if self.collection_ids:
+            builder.collection_ids = self.collection_ids
 
         return builder
